@@ -4,6 +4,7 @@
 #include "spvm_native.h"
 
 #include <openssl/ssl.h>
+#include <openssl/err.h>
 
 static const char* FILE_NAME = "Net/SSLeay/X509_VERIFY_PARAM.c";
 
@@ -50,7 +51,20 @@ int32_t SPVM__Net__SSLeay__X509_VERIFY_PARAM__set1_host(SPVM_ENV* env, SPVM_VALU
   
   X509_VERIFY_PARAM* x509_verify_param = env->get_pointer(env, stack, obj_self);
   
-  X509_VERIFY_PARAM_set1_host(x509_verify_param, name, namelen);
+  int32_t status = X509_VERIFY_PARAM_set1_host(x509_verify_param, name, namelen);
+  
+  if (!(status == 1)) {
+    int64_t ssl_error = ERR_peek_last_error();
+    
+    char* ssl_error_string = env->get_stack_tmp_buffer(env, stack);
+    ERR_error_string_n(ssl_error, ssl_error_string, SPVM_NATIVE_C_STACK_TMP_BUFFER_SIZE);
+    
+    env->die(env, stack, "[OpenSSL Error]X509_VERIFY_PARAM_set1_host failed:%s.", ssl_error_string, __func__, FILE_NAME, __LINE__);
+    
+    int32_t error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
+    
+    return error_id;
+  }
   
   return 0;
 }
